@@ -1,4 +1,4 @@
-use crate::{Class, Tag, Tagged};
+use crate::{Class, DerAutoDerive, Error, Tag, Tagged};
 use core::marker::PhantomData;
 
 mod builder;
@@ -39,63 +39,91 @@ impl TagKind for Explicit {}
 /// # Notes
 ///
 /// `CLASS` must be between 0 and 4. See [`Class`] for possible values for the `CLASS` parameter.
+/// Constants from this class can be used, but they must be wrapped in braces due to
+/// [Rust syntax for generics](https://doc.rust-lang.org/reference/items/generics.html)
+/// (see example below).
 ///
 /// # Examples
 ///
 /// To parse a `[APPLICATION 0] EXPLICIT INTEGER` object:
 ///
 /// ```rust
-/// use asn1_rs::{Explicit, FromBer, Integer, TaggedValue};
+/// use asn1_rs::{Class, Error, Explicit, FromBer, Integer, TaggedValue};
 ///
 /// let bytes = &[0x60, 0x03, 0x2, 0x1, 0x2];
 ///
 /// // If tagged object is present (and has expected tag), parsing succeeds:
-/// let (_, tagged) = TaggedValue::<Integer, Explicit, 0b01, 0>::from_ber(bytes).unwrap();
+/// let (_, tagged) =
+///     TaggedValue::<Integer, Error, Explicit, {Class::APPLICATION}, 0>::from_ber(bytes)
+///         .unwrap();
 /// assert_eq!(tagged, TaggedValue::explicit(Integer::from(2)));
 /// ```
 #[derive(Debug, PartialEq)]
-pub struct TaggedValue<T, TagKind, const CLASS: u8, const TAG: u32> {
+pub struct TaggedValue<T, E, TagKind, const CLASS: u8, const TAG: u32> {
     pub(crate) inner: T,
 
     tag_kind: PhantomData<TagKind>,
+    _e: PhantomData<E>,
 }
 
-impl<T, TagKind, const CLASS: u8, const TAG: u32> TaggedValue<T, TagKind, CLASS, TAG> {
+impl<T, E, TagKind, const CLASS: u8, const TAG: u32> TaggedValue<T, E, TagKind, CLASS, TAG> {
     /// Consumes the `TaggedParser`, returning the wrapped value.
     #[inline]
     pub fn into_inner(self) -> T {
         self.inner
     }
+
+    /// Return the (outer) tag of this object
+    pub const fn tag(&self) -> Tag {
+        Self::TAG
+    }
+
+    /// Return the (outer) class of this object
+    #[inline]
+    pub const fn class(&self) -> u8 {
+        CLASS
+    }
 }
 
-impl<T, const CLASS: u8, const TAG: u32> TaggedValue<T, Explicit, CLASS, TAG> {
+impl<T, E, const CLASS: u8, const TAG: u32> TaggedValue<T, E, Explicit, CLASS, TAG> {
     /// Constructs a new `EXPLICIT TaggedParser` with the provided value
     #[inline]
     pub const fn explicit(inner: T) -> Self {
         TaggedValue {
             inner,
             tag_kind: PhantomData,
+            _e: PhantomData,
         }
     }
 }
 
-impl<T, const CLASS: u8, const TAG: u32> TaggedValue<T, Implicit, CLASS, TAG> {
+impl<T, E, const CLASS: u8, const TAG: u32> TaggedValue<T, E, Implicit, CLASS, TAG> {
     /// Constructs a new `IMPLICIT TaggedParser` with the provided value
     #[inline]
     pub const fn implicit(inner: T) -> Self {
         TaggedValue {
             inner,
             tag_kind: PhantomData,
+            _e: PhantomData,
         }
     }
 }
 
-impl<T, TagKind, const CLASS: u8, const TAG: u32> AsRef<T> for TaggedValue<T, TagKind, CLASS, TAG> {
+impl<T, E, TagKind, const CLASS: u8, const TAG: u32> AsRef<T>
+    for TaggedValue<T, E, TagKind, CLASS, TAG>
+{
     fn as_ref(&self) -> &T {
         &self.inner
     }
 }
 
-impl<T, TagKind, const CLASS: u8, const TAG: u32> Tagged for TaggedValue<T, TagKind, CLASS, TAG> {
+impl<T, E, TagKind, const CLASS: u8, const TAG: u32> Tagged
+    for TaggedValue<T, E, TagKind, CLASS, TAG>
+{
     const TAG: Tag = Tag(TAG);
 }
+
+impl<'a, T, E, TagKind, const CLASS: u8, const TAG: u32> DerAutoDerive
+    for TaggedValue<T, E, TagKind, CLASS, TAG>
+{
+}
